package com.docmosis.example;


import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

import com.docmosis.SystemManager;
import com.docmosis.document.DocumentProcessor;
import com.docmosis.document.ProcessingException;
import com.docmosis.document.converter.ConversionException;
import com.docmosis.document.converter.ConversionFormat;
import com.docmosis.document.converter.ConversionInstruction;
import com.docmosis.document.converter.NoConvertersRunningException;
import com.docmosis.document.converter.pool.ConverterPoolException;
import com.docmosis.template.TemplateStoreException;
import com.docmosis.template.population.CompoundDataProvider;
import com.docmosis.template.population.DataProvider;
import com.docmosis.template.population.MemoryDataProvider;
import com.docmosis.template.population.ReflectiveDataProvider;
import com.docmosis.template.population.render.FieldDetails;
import com.docmosis.template.population.render.FieldRenderer;
import com.docmosis.template.population.render.RenderedField;
import com.docmosis.template.store.DropStoreHelper;
import com.docmosis.template.store.TemplateContext;
import com.docmosis.template.store.TemplateIdentifier;
import com.docmosis.template.store.TemplateStoreFactory;
import com.docmosis.util.FileUtilities;
import com.docmosis.util.logging.LogManager;
import com.docmosis.util.logging.Logger;

/**
 * Example code demonstrating some of the ways that Docmosis can be used.
 */
public class ResumeExample
{
	private static final Logger log = LogManager.getLogger(ResumeExample.class);

	private static void prepareTemplates()
	throws IOException, ConversionException, ConverterPoolException, TemplateStoreException, NoConvertersRunningException
	{
		DropStoreHelper storeHelper = new DropStoreHelper(TemplateStoreFactory.getStore());
		storeHelper.process(new File("."), false);
		log.info("Template(s) loaded into store");
	}
	
	private static void prepareRenderer(ConversionInstruction instruction)
	{
		
		// The following code adds a renderer to the Conversion Instructionto control the 
		// display behaviour of specific fields.  This renderer is registered against "renderer1" which
		// is referenced by the template.
		instruction.setRenderer("renderer1", new ResumeFieldRenderer());
		
		// the renderer above could also be set as a system default meaning it is available
		// to all document runs (not just those using this instruction).
		//DocumentProcessor.setDefaultRenderer("renderer1", new ResumeFieldRenderer());
	}
	
	private static void renderCV(DataProvider dp) 
	  throws IOException, ConversionException, ConverterPoolException, TemplateStoreException, ProcessingException
	{
		TemplateIdentifier templateId = new TemplateIdentifier("resumeTemplate.doc", 
				new TemplateContext(""));
		
		ConversionInstruction instruction = new ConversionInstruction();
		instruction.setConversionFormats(new ConversionFormat[]{
//				ConversionFormat.FORMAT_ODF,
				ConversionFormat.FORMAT_WORD,
				ConversionFormat.FORMAT_PDF,
		});
		
		prepareRenderer(instruction);
		
		String personName = dp.getString("personName");
		personName = personName == null ? "unknown" : personName;
	
		// the name of the documents inside the zip file
		instruction.setOutputFileName(personName+"cv");
		
		// the file in which to store the result.
		FileOutputStream streamTo = new FileOutputStream("resume.zip");
		try {
			long start = System.currentTimeMillis();
			DocumentProcessor.renderDoc(templateId, dp, instruction, streamTo);
			log.info("resume.zip generated in " + (System.currentTimeMillis() - start) + " milliseconds");
		} finally {
			FileUtilities.close(streamTo);
		}
	}
	
	
	/**
	 * Build a DataProvider that reflects a CVRecord Object
	 * 
	 * @return a new data provider that will provide data for the template.
	 */
	private static DataProvider getPersonData()
	{
		String name = "James Jowie";
		
		Calendar c = Calendar.getInstance();
		c.set(Calendar.YEAR, 1966);
		c.set(Calendar.MONTH, Calendar.JANUARY);
		c.set(Calendar.DAY_OF_MONTH, 15);
		Person person = new Person(name, c.getTime());
		
		person.setSkills(new Skill[] {
				new Skill(3.0f, "Java Programming (J2EE, Swing)"),
				new Skill(1.5f, "Database Development (Oracle, Sybase)"),
				new Skill(2.0f, "Team Leading"),
				new Skill(7.0f, "CV re-writing"),
		});
		
		person.setEducation(new Education("Feb 1977", "Nov 1979", "Vancouver Labs", 
				new String[]{"BA Social Engineering", "BA of BS", "Punchcard Mastery"}));
		
		person.setInterests("Running, swimming, candle-lit dinners for three");
		
		ArrayList employmentHistory = new ArrayList();
		employmentHistory
				.add(new Job(
						"1980",
						"1984",
						"Mintwell",
						"General Hand",
						"Manual tasks since it was early days for computing.\nMostly photocopying"
								+ " and fetching coffee.  In fact I was affectionately known as 'gofer' during this time."));
		employmentHistory
				.add(new Job(
						"1984",
						"1985",
						"IBMB",
						"Punch Card Legend",
						"Tried my hand at computer programming with the newest punchcard technology.  I " +
						"got plenty of time to read magazines and drink coffee while the programs compiled"));
		employmentHistory
		.add(new Job(
				"1986",
				"1987",
				"MacroSAFT",
				"Team Superior",
				"Now I was hitting my stride, writing hundreds of lines (of comments) and running a " +
				"dilligent team of staff (myself included (myself only))"));
		
		person.setEmploymentHistory(employmentHistory);
		person.setDriversLicense(Boolean.TRUE);
		
		Reference[] references = new Reference[] {
				new Reference("James Blundo", "552 444 555"),	
				new Reference("Jillie Dava", "+44 022 22221"),	
		};
		CVDetails cvDetails = new CVDetails(new Date(), person, references);
		
		ReflectiveDataProvider rdp = ReflectiveDataProvider.getInstance(cvDetails, false);
		MemoryDataProvider overridingValues = new MemoryDataProvider();
		
		// plug the person name in at the top level of the data (in addition to the reflective data)
		overridingValues.setString("personName", name);
		overridingValues.setString("address", "10 the burbs\nSuburbia 6101");
		overridingValues.setString("objective", "To be the best programmer in the world without the need to go through years of" +
				" gaining experience");
		
		CompoundDataProvider cdp = new CompoundDataProvider();
		cdp.addDataObject(overridingValues);
		cdp.addDataObject(rdp);

		return cdp;
	}
	
	/**
	 * @param args
	 * @throws ProcessingException 
	 * @throws TemplateStoreException 
	 * @throws ConverterPoolException 
	 * @throws ConversionException 
	 * @throws IOException 
	 * @throws NoConvertersRunningException
	 */
	public static void main(String[] args)
	throws IOException, ConversionException, ConverterPoolException, TemplateStoreException, ProcessingException, NoConvertersRunningException
	{
		if (args != null && args.length > 0) {
			printUsageAndExit();
		}
		
		// initialise the system based on configuration
		// this only needs to be done once at the very start of Docmosis use.
    	SystemManager.initialise();

    	// load the data from the sampleData.txt file
    	DataProvider dp = getPersonData();

		try {
			// load any templates in the current directory into the template store
			prepareTemplates();
			
			// run a single document render
			renderCV(dp);
		} finally {
			// shutdown the system
			SystemManager.release();
		}
	}

	private static void printUsageAndExit()
	{
		System.err.println("usage: java " + ResumeExample.class.getName() + " [-h]");
		System.err.println("  -h        : print this usage");
		System.err.println("  <no args> : render a CV (creates resume.zip)");
		System.exit(1);
	}
	
	
	
	public static class CVDetails
	{
		Date cvDate;
		Person person;
		Reference[] references;
		
		public CVDetails(Date cvDate, Person person, Reference[] references)
		{
			super();
			this.cvDate = cvDate;
			this.person = person;
			this.references = references;
		}
		public Date getCvDate()
		{
			return cvDate;
		}
		public Person getPerson()
		{
			return person;
		}
		public Reference[] getReferences() 
		{
			return references;
		}
	}
	
	public static class Person
	{
		String name;
		Date dob;
		Skill[] skills;
		List employmentHistory; // list of Jobs
		Education education;
		String interests;
		boolean australianCitizen;
		Boolean driversLicense;
		
		public Person(String name, Date dob)
		{
			super();
			this.name = name;
			this.dob = dob;
		}

		public Date getDob()
		{
			return dob;
		}

		public void setDob(Date dob)
		{
			this.dob = dob;
		}

		public List getEmploymentHistory()
		{
			return employmentHistory;
		}

		public void setEmploymentHistory(List employmentHistory)
		{
			this.employmentHistory = employmentHistory;
		}

		public String getName()
		{
			return name;
		}

		public void setName(String name)
		{
			this.name = name;
		}

		public Skill[] getSkills()
		{
			return skills;
		}

		public void setSkills(Skill[] skills)
		{
			this.skills = skills;
		}

		public Education getEducation()
		{
			return education;
		}

		public void setEducation(Education education)
		{
			this.education = education;
		}

		public String getInterests()
		{
			return interests;
		}

		public void setInterests(String interests)
		{
			this.interests = interests;
		}

		public boolean isAustralianCitizen()
		{
			return australianCitizen;
		}

		public void setAustralianCitizen(boolean australianCitizen)
		{
			this.australianCitizen = australianCitizen;
		}

		public Boolean getDriversLicense()
		{
			return driversLicense;
		}

		public void setDriversLicense(Boolean driversLicense)
		{
			this.driversLicense = driversLicense;
		}
		
		
	}

	public static class Education
	{
		private String from;
		private String to;
		private String location;		
		private String[] history;
		
		public Education(String from, String to, String location, String[] history)
		{
			super();
			this.from = from;
			this.to = to;
			this.location = location;
			this.history = history;
		}
		public String getFrom()
		{
			return from;
		}
		public String[] getHistory()
		{
			return history;
		}
		public String getTo()
		{
			return to;
		}
		public String getLocation()
		{
			return location;
		}
	}
	
	public static class Skill
	{
		float yearsExperience;
		String skillName;
		public Skill(float yearsExperience, String skillName)
		{
			super();
			this.yearsExperience = yearsExperience;
			this.skillName = skillName;
		}
		public String getSkillName()
		{
			return skillName;
		}
		public float getYearsExperience()
		{
			return yearsExperience;
		}
		
	}
	
	public static class Job
	{
		String start;
		String end;
		String companyName;
		String jobDesc;
		String title;
		public Job(String start, String end, String companyName, String title, String jobDesc)
		{
			super();
			this.start = start;
			this.end = end;
			this.companyName = companyName;
			this.title = title;
			this.jobDesc = jobDesc;
		}
		public String getCompanyName()
		{
			return companyName;
		}
		public String getEnd()
		{
			return end;
		}
		public String getJobDesc()
		{
			return jobDesc;
		}
		public String getStart()
		{
			return start;
		}
		public String getTitle()
		{ 
			return title;
		}
	}
	
	public static class Reference
	{
		private String name;
		private String phone;
		
		public Reference(String name, String phone)
		{
			super();
			this.name = name;
			this.phone = phone;
		}

		public String getName()
		{
			return name;
		}

		public String getPhone()
		{
			return phone;
		}
		
	}
	
	/**
	 * An example implementation of a FieldRenderer.  This example formats any 
	 * date data with a date formatter and applies cell colouring to the experience 
	 * table depending on years of experience.
	 */
	private static class ResumeFieldRenderer implements FieldRenderer
	{
		final SimpleDateFormat dateFormatter = new SimpleDateFormat("dd/MM/yyyy");
		
		public RenderedField render(FieldDetails field, RenderedField renderedField)
		{
			if (log.isDebugEnabled()) {
				log.debug("field=" + field.getFieldName()+ 
						" val=" + field.getFieldValue()+
						" rendererName=" + field.getRendererName()+
						" row=" + field.getRowNum()+ 
						" valueClass=" + field.getValueClass()+
						" valueObject=" + field.getValueObject()+
						" containerClass=" + field.getContainerClass()+
						" containerObject=" + field.getContainerObject());
			}
			
			String colour = null;
			boolean embolden = false;
			
			if ("yearsExperience".equals(field.getFieldName())) {
				float yearsExperince = ((Float)field.getValueObject()).floatValue();
				if (yearsExperince <= 1.5f) {
					// don't highlight
				} else if (yearsExperince <= 3.0f) {
					// interesting
					colour = "#77AA77";
				} else {
					// notable, set colour and make bold.
					colour = "#77EE77";
					embolden = true;
				}
			}

			String value = null;
			if (field.getValueClass() != null && field.getValueClass().equals(Date.class)) {
				synchronized(dateFormatter) {
					// synchronized since SimpleDateFormat objects aren't thread safe.
					value = dateFormatter.format((Date)field.getValueObject());
				}
			}
			
    		RenderedField rendered = renderedField;
    		rendered.setCellBackgroundColour(colour);
    		rendered.setValue(value);
    		rendered.setBold(embolden);
    		
    		return rendered;
		}

	}
}
