package com.docmosis.example;


import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;
import java.sql.Time;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.Map;

import com.docmosis.SystemManager;
import com.docmosis.document.DocumentProcessor;
import com.docmosis.document.ProcessingException;
import com.docmosis.document.converter.ConversionException;
import com.docmosis.document.converter.ConversionFormat;
import com.docmosis.document.converter.ConversionInstruction;
import com.docmosis.document.converter.NoConvertersRunningException;
import com.docmosis.document.converter.pool.ConverterPoolException;
import com.docmosis.template.TemplateStoreException;
import com.docmosis.template.population.DataProviderFactory;
import com.docmosis.template.population.DataProviderGrouping;
import com.docmosis.template.population.DataProviderGroupingException;
import com.docmosis.template.population.DataProviderSQLGrouping;
import com.docmosis.template.population.Discriminator;
import com.docmosis.template.population.MemoryDataProvider;
import com.docmosis.template.population.NoSuchColumnNameException;
import com.docmosis.template.population.SQLDataProvider;
import com.docmosis.template.store.StoreHelper;
import com.docmosis.template.store.TemplateContext;
import com.docmosis.template.store.TemplateIdentifier;
import com.docmosis.template.store.TemplateStoreFactory;
import com.docmosis.util.FileUtilities;
import com.docmosis.util.logging.LogManager;
import com.docmosis.util.logging.Logger;

/**
 * Example code demonstrating some of the ways that Docmosis can be used.
 */
public class SQLExample
{
	private static final Logger log = LogManager.getLogger(SQLExample.class);

	private static void prepareTemplates()
	throws IOException, ConversionException, ConverterPoolException, TemplateStoreException, NoConvertersRunningException
	{
        StoreHelper.storeTemplate(
                "SqlExampleTemplate.odt", new File("SqlExampleTemplate.odt"),
                TemplateStoreFactory.getStore());
                
		log.info("Template(s) loaded into store");
	}
	
	private static void renderReport() 
	throws IOException, ConversionException, ConverterPoolException, TemplateStoreException,
	ProcessingException, NoSuchColumnNameException, SQLException, DataProviderGroupingException,
	ClassNotFoundException
	{
		TemplateIdentifier templateId = new TemplateIdentifier("SqlExampleTemplate.odt", 
				new TemplateContext(""));
		
		ConversionInstruction instruction = new ConversionInstruction();
		instruction.setConversionFormats(new ConversionFormat[]{
				ConversionFormat.FORMAT_PDF,
		});
		
		// In this example we are going to re-use the same data provider for several
		// separate renders, so we disable the auto cleanup feature via the instruction
		// and handle the cleanup ourselves.  If the data provider contains image streams
		// then this would not work since the image streams can only be read once.
		instruction.setCleanupDataProvider(false);
	
		// the name of the documents inside the zip file if creating a zip
//		instruction.setOutputFileName("SqlExample");
		
		// the file in which to store the result.
		FileOutputStream streamTo = new FileOutputStream("SqlExample.pdf");

		// obtain the data
    	MemoryDataProvider dp = getSQLData();

    	try {
	    	
	    	// debug the data provider if desired
	    	if (log.isDebugEnabled()) {
	    		log.debug(dp.toDebugString());
	    	}

			// instead of creating a zip, we render three separate documents in three
			// separate calls.  Typically this is not how you would do it, but examples
			// are for learning.. ;)
			long start = System.currentTimeMillis();
			DocumentProcessor.renderDoc(templateId, dp, instruction, streamTo);
			log.info("SqlExample.pdf written");
			
			FileUtilities.close(streamTo);
			instruction.setConversionFormats(new ConversionFormat[]{ConversionFormat.FORMAT_WORD});
			streamTo = new FileOutputStream("SqlExample.doc");
			DocumentProcessor.renderDoc(templateId, dp, instruction, streamTo);
			log.info("SqlExample.doc written");			

			FileUtilities.close(streamTo);
			instruction.setConversionFormats(new ConversionFormat[]{ConversionFormat.FORMAT_ODT});
			streamTo = new FileOutputStream("SqlExample.odt");
			DocumentProcessor.renderDoc(templateId, dp, instruction, streamTo);
			log.info("SqlExample.odt written");

			FileUtilities.close(streamTo);
			instruction.setConversionFormats(new ConversionFormat[]{ConversionFormat.FORMAT_HTML});
			streamTo = new FileOutputStream("SqlExample.html");
			DocumentProcessor.renderDoc(templateId, dp, instruction, streamTo);
			log.info("SqlExample.html written");

			FileUtilities.close(streamTo);
			instruction.setConversionFormats(new ConversionFormat[]{ConversionFormat.FORMAT_TEXT});
			streamTo = new FileOutputStream("SqlExample.txt");
			DocumentProcessor.renderDoc(templateId, dp, instruction, streamTo);
			log.info("SqlExample.txt written");

			log.info("SqlExample generated in 5 separate renders in " + (System.currentTimeMillis() - start) + " milliseconds");
		} finally {
			FileUtilities.close(streamTo);
			dp.cleanup();
		}
	}
	
	
	/**
	 * Build a DataProvider that has extracted data from an SQL query.
	 * 
	 * @return a new data provider that will provide data for the template.
	 * @throws DataProviderGroupingException 
	 * @throws SQLException 
	 * @throws NoSuchColumnNameException 
	 */
	private static MemoryDataProvider getSQLData()
	throws NoSuchColumnNameException, SQLException, DataProviderGroupingException,
	ClassNotFoundException
	{
		
		String sql = 
	      "select id, heading, district_id, district_name, category, male_count, female_count, " +
	      "district_male_total, district_female_total, district_total from campusSummary";

		ResultSet rs = executeQuery(sql);

		// This section defines how to group data into a hierarchy to match the query data to the template.
		// Here we define 3 levels of grouping: "data" which contains "districts" which contain "categories"
        DataProviderGrouping[] grouping = new DataProviderSQLGrouping[3];
        grouping[0] = new DataProviderSQLGrouping("data", new Discriminator("ID"), new String[]{"HEADING", "NDISTRICTS","MALETOTAL", "MALEPCT", "FEMALETOTAL", "FEMALEPCT", "TOTAL"});
        grouping[1] = new DataProviderSQLGrouping("districts", new Discriminator("DISTRICT_ID"), new String[] { "DISTRICT_NAME", "DISTRICT_MALE_TOTAL","DISTRICT_FEMALE_TOTAL","DISTRICT_TOTAL" });
        grouping[2] = new DataProviderSQLGrouping("categories", new Discriminator("CATEGORY"), new String[] { "MALE_COUNT","FEMALE_COUNT","DISTRICT_COUNT" });
		
        SQLDataProvider dp = DataProviderFactory.createDataProvider(rs, grouping);
		
		// plug in other data for the template
		dp.setString("reportDate",
				new SimpleDateFormat("dd/MM/yyyy HH:mm").format(new Date()));

		return dp;
	}
	
	private static ResultSet executeQuery(String sql)
	{
		// we don't really run a query, we just fake it and return a ResultSet implementation
		// that has the data we want.  The alternative of actually using a database is considered
		// too onerous for this demonstration.
		
		return new DummyResultSet();
	}
	
	/**
	 * Main Entry point
	 */
	public static void main(String[] args)
	throws IOException, ConversionException, ConverterPoolException, TemplateStoreException,
	ProcessingException, NoConvertersRunningException, NoSuchColumnNameException, SQLException,
	DataProviderGroupingException, ClassNotFoundException
	{
		if (args != null && args.length > 0) {
			printUsageAndExit();
		}
		
		// initialise the system based on configuration
		// this only needs to be done once at the very start of Docmosis use.
    	SystemManager.initialise();

		try {
			// load any templates in the current directory into the template store
			prepareTemplates();
			
			// run a single document render
			renderReport();
		} finally {
			// shutdown the system
			SystemManager.release();
		}
	}

	private static void printUsageAndExit()
	{
		System.err.println("usage: java " + SQLExample.class.getName() + " [-h]");
		System.err.println("  -h        : print this usage");
		System.err.println("  <no args> : render a CV (creates SqlExample.pdf)");
		System.exit(1);
	}
	
	
	
	/**
	 * This class simply pretends to return a ResultSet from the query.  It returns
	 * random data fitting the bill of the demonstration.
	 */
	private static class DummyResultSet implements ResultSet
	{
		private static final String[] COLUMNS = new String[]{
			"ID","HEADING","DISTRICT_ID","DISTRICT_NAME","CATEGORY","MALE_COUNT","FEMALE_COUNT","DISTRICT_COUNT",
			"DISTRICT_MALE_TOTAL","DISTRICT_FEMALE_TOTAL","DISTRICT_TOTAL", "NDISTRICTS", "MALETOTAL", "MALEPCT", 
			"FEMALETOTAL", "FEMALEPCT", "TOTAL"
		};

		private static final String[] COLUMN_CLASSES = new String[]{
			"java.lang.Long","java.lang.String","java.lang.Long","java.lang.String","java.lang.String","java.lang.Integer","java.lang.Integer","java.lang.Integer",
			"java.lang.Integer","java.lang.Integer","java.lang.Integer","java.lang.Integer","java.lang.Integer","java.lang.Double",
			"java.lang.Integer","java.lang.Double","java.lang.Integer"
//			"id","heading","district_id","district_name","year_level","male_count","female_count","student_count","district_male_total","district_female_total","district_total"
		};		
		
		private static final String[][] DATA = generateData();

		private static final DummyMetaData metaData = new DummyMetaData(COLUMNS, COLUMN_CLASSES);
		
		private int currentRow =-1;
		
		public DummyResultSet()
		{
		}

		private static String[][] generateData()
		{
			
		    final String[] districts = {"Wembley", "Booragoon", "Zinnafex", "Plaistowe", "Florida",
		    		"New York", "Greater London", "Montreal", "Nanda", "Manchester",
		    		"Birmingham", "Denmark", "Paris", "Barcelona"};
			final int nDistricts = districts.length;
			final String[] categories = {"Youth", "Adult", "Senior"};
			final int nCategories = categories.length;
			final int reportID = 1;
			final String reportTitle = "Districts Population Report";
			
			int maleTotal = 0;
			int femaleTotal = 0;
			
			String[][] results = new String[nDistricts * nCategories][];
			
			for(int i=0 ; i<nDistricts; i++) {
				int districtMale = 0;
				int districtFemale = 0;
				int districtTotal = 0;
				
				for(int j=0; j < nCategories; j++) {
					int maleCount = (int)(Math.random() * 3100.0);
					int femaleCount = (int)(Math.random() * 3100.0);
					int total = maleCount + femaleCount;
					
					districtMale += maleCount;
					districtFemale += femaleCount;
					districtTotal += total;
					
					results[i * nCategories + j] = new String[] {String.valueOf(reportID), reportTitle, String.valueOf(i), districts[i],
							categories[j],
							String.valueOf(maleCount), String.valueOf(femaleCount), String.valueOf(total),
							"","","", // totals to be filled in consistently later
							String.valueOf(nDistricts),
							"", "", "", "", ""};// totals to be filled in consistently later
				}
				
				// go back and plug in district totals
				for(int j=0; j<nCategories; j++) {
					results[i * nCategories + j][8] = String.valueOf(districtMale);
					results[i * nCategories + j][9] = String.valueOf(districtFemale);
					results[i * nCategories + j][10] = String.valueOf(districtTotal);
				}
				
				maleTotal += districtMale;
				femaleTotal += districtFemale;
			}

			// fill in final totals
			for(int i=0 ; i < nDistricts * nCategories; i++) {
				int total = maleTotal + femaleTotal;
				results[i][12] = String.valueOf(maleTotal);
				results[i][13] = String.valueOf(asPct((double)maleTotal/total));
				results[i][14] = String.valueOf(femaleTotal);
				results[i][15] = String.valueOf(asPct((double)femaleTotal/total));
				results[i][16] = String.valueOf(total);
			}
			
			// debug
			if (log.isDebugEnabled()) {
				StringBuffer sb = new StringBuffer(128);
				for(int i=0; i<results.length; i++) {
					sb.setLength(0);
					for(int j=0; j<results[i].length; j++) {
						if (j > 0) {
							sb.append(',');
						}
						sb.append(results[i][j]);
					}
					System.out.println(sb.toString());
				}
			}
			
			return results;
		}
		
		private static String asPct(double value) 
		{
			return (((int)(value * 1000.0))/10.0) + "%";
		}
		
		public String getString(int columnIndex) throws SQLException
		{
			return DATA[currentRow][columnIndex-1];
		}

		public boolean next() throws SQLException
		{
			if (currentRow < DATA.length -1) {
				currentRow ++;
				return true;
			} else {
				return false;
			}
		}


		public String getString(String columnName) throws SQLException
		{
			for(int i=0;i<COLUMNS.length;i++) {
				if (COLUMNS[i].equals(columnName)) {
					return getString(i+1);
				}
			}
			throw new SQLException("Column not known:" + columnName);
		}

		public boolean absolute(int row) throws SQLException
		{
			return false;
		}

		public void afterLast() throws SQLException{}
		public void beforeFirst() throws SQLException{}
		public void cancelRowUpdates() throws SQLException{}
		public void clearWarnings() throws SQLException{}
		public void close() throws SQLException{}
		public void deleteRow() throws SQLException{}
		public int findColumn(String columnName) throws SQLException
		{
			// not implemented
			return 0;
		}

		public boolean first() throws SQLException
		{
			// not implemented
			return false;
		}

		public Array getArray(int i) throws SQLException
		{
			// not implemented
			return null;
		}

		public Array getArray(String colName) throws SQLException
		{
			// not implemented
			return null;
		}

		public InputStream getAsciiStream(int columnIndex) throws SQLException
		{
			// not implemented
			return null;
		}

		public InputStream getAsciiStream(String columnName) throws SQLException
		{
			// not implemented
			return null;
		}

		public BigDecimal getBigDecimal(int columnIndex, int scale) throws SQLException
		{
			// not implemented
			return null;
		}

		public BigDecimal getBigDecimal(int columnIndex) throws SQLException
		{
			// not implemented
			return null;
		}

		public BigDecimal getBigDecimal(String columnName, int scale) throws SQLException
		{
			// not implemented
			return null;
		}

		public BigDecimal getBigDecimal(String columnName) throws SQLException
		{
			// not implemented
			return null;
		}

		public InputStream getBinaryStream(int columnIndex) throws SQLException
		{
			// not implemented
			return null;
		}

		public InputStream getBinaryStream(String columnName) throws SQLException
		{
			// not implemented
			return null;
		}

		public Blob getBlob(int i) throws SQLException
		{
			// not implemented
			return null;
		}

		public Blob getBlob(String colName) throws SQLException
		{
			// not implemented
			return null;
		}

		public boolean getBoolean(int columnIndex) throws SQLException
		{
			// not implemented
			return false;
		}

		public boolean getBoolean(String columnName) throws SQLException
		{
			// not implemented
			return false;
		}

		public byte getByte(int columnIndex) throws SQLException
		{
			// not implemented
			return 0;
		}

		public byte getByte(String columnName) throws SQLException
		{
			// not implemented
			return 0;
		}

		public byte[] getBytes(int columnIndex) throws SQLException
		{
			// not implemented
			return null;
		}

		public byte[] getBytes(String columnName) throws SQLException
		{
			// not implemented
			return null;
		}

		public Reader getCharacterStream(int columnIndex) throws SQLException
		{
			// not implemented
			return null;
		}

		public Reader getCharacterStream(String columnName) throws SQLException
		{
			// not implemented
			return null;
		}

		public Clob getClob(int i) throws SQLException
		{
			// not implemented
			return null;
		}

		public Clob getClob(String colName) throws SQLException
		{
			// not implemented
			return null;
		}

		public int getConcurrency() throws SQLException
		{
			// not implemented
			return 0;
		}

		public String getCursorName() throws SQLException
		{
			// not implemented
			return null;
		}

		public java.sql.Date getDate(int columnIndex, Calendar cal) throws SQLException
		{
			// not implemented
			return null;
		}

		public java.sql.Date getDate(int columnIndex) throws SQLException
		{
			// not implemented
			return null;
		}

		public java.sql.Date getDate(String columnName, Calendar cal) throws SQLException
		{
			// not implemented
			return null;
		}

		public java.sql.Date getDate(String columnName) throws SQLException
		{
			// not implemented
			return null;
		}

		public double getDouble(int columnIndex) throws SQLException
		{
			// not implemented
			return 0;
		}

		public double getDouble(String columnName) throws SQLException
		{
			// not implemented
			return 0;
		}

		public int getFetchDirection() throws SQLException
		{
			// not implemented
			return 0;
		}

		public int getFetchSize() throws SQLException
		{
			// not implemented
			return 0;
		}

		public float getFloat(int columnIndex) throws SQLException
		{
			// not implemented
			return 0;
		}

		public float getFloat(String columnName) throws SQLException
		{
			// not implemented
			return 0;
		}

		public int getInt(int columnIndex) throws SQLException
		{
			// not implemented
			return 0;
		}

		public int getInt(String columnName) throws SQLException
		{
			// not implemented
			return 0;
		}

		public long getLong(int columnIndex) throws SQLException
		{
			// not implemented
			return 0;
		}

		public long getLong(String columnName) throws SQLException
		{
			// not implemented
			return 0;
		}

		public ResultSetMetaData getMetaData() throws SQLException
		{
			return metaData;
		}

		public Object getObject(int i, Map map) throws SQLException
		{
			// not implemented
			return null;
		}

		public Object getObject(int columnIndex) throws SQLException
		{
			return getString(columnIndex);
		}

		public Object getObject(String colName, Map map) throws SQLException
		{
			// not implemented
			return null;
		}

		public Object getObject(String columnName) throws SQLException
		{
			return getString(columnName);
		}

		public Ref getRef(int i) throws SQLException
		{
			// not implemented
			return null;
		}

		public Ref getRef(String colName) throws SQLException
		{
			// not implemented
			return null;
		}

		public int getRow() throws SQLException
		{
			// not implemented
			return 0;
		}

		public short getShort(int columnIndex) throws SQLException
		{
			// not implemented
			return 0;
		}

		public short getShort(String columnName) throws SQLException
		{
			// not implemented
			return 0;
		}

		public Statement getStatement() throws SQLException
		{
			// not implemented
			return null;
		}

		public Time getTime(int columnIndex, Calendar cal) throws SQLException
		{
			// not implemented
			return null;
		}

		public Time getTime(int columnIndex) throws SQLException
		{
			// not implemented
			return null;
		}

		public Time getTime(String columnName, Calendar cal) throws SQLException
		{
			// not implemented
			return null;
		}

		public Time getTime(String columnName) throws SQLException
		{
			// not implemented
			return null;
		}

		public Timestamp getTimestamp(int columnIndex, Calendar cal) throws SQLException
		{
			// not implemented
			return null;
		}

		public Timestamp getTimestamp(int columnIndex) throws SQLException
		{
			// not implemented
			return null;
		}

		public Timestamp getTimestamp(String columnName, Calendar cal) throws SQLException
		{
			// not implemented
			return null;
		}

		public Timestamp getTimestamp(String columnName) throws SQLException
		{
			// not implemented
			return null;
		}

		public int getType() throws SQLException
		{
			// not implemented
			return 0;
		}

		public InputStream getUnicodeStream(int columnIndex) throws SQLException
		{
			// not implemented
			return null;
		}

		public InputStream getUnicodeStream(String columnName) throws SQLException
		{
			// not implemented
			return null;
		}

		public URL getURL(int columnIndex) throws SQLException
		{
			// not implemented
			return null;
		}

		public URL getURL(String columnName) throws SQLException
		{
			// not implemented
			return null;
		}

		public SQLWarning getWarnings() throws SQLException
		{
			// not implemented
			return null;
		}

		public void insertRow() throws SQLException{}

		public boolean isAfterLast() throws SQLException
		{
			// not implemented
			return false;
		}

		public boolean isBeforeFirst() throws SQLException
		{
			// not implemented
			return false;
		}

		public boolean isFirst() throws SQLException
		{
			// not implemented
			return false;
		}

		public boolean isLast() throws SQLException
		{
			// not implemented
			return false;
		}

		public boolean last() throws SQLException
		{
			// not implemented
			return false;
		}

		public void moveToCurrentRow() throws SQLException{}

		public void moveToInsertRow() throws SQLException{}

		public boolean previous() throws SQLException
		{
			// not implemented
			return false;
		}

		public void refreshRow() throws SQLException{}

		public boolean relative(int rows) throws SQLException
		{
			// not implemented
			return false;
		}

		public boolean rowDeleted() throws SQLException
		{
			// not implemented
			return false;
		}

		public boolean rowInserted() throws SQLException
		{
			// not implemented
			return false;
		}

		public boolean rowUpdated() throws SQLException
		{
			// not implemented
			return false;
		}

		public void setFetchDirection(int direction) throws SQLException{}
		public void setFetchSize(int rows) throws SQLException{}
		public void updateArray(int columnIndex, Array x) throws SQLException{}
		public void updateArray(String columnName, Array x) throws SQLException{}
		public void updateAsciiStream(int columnIndex, InputStream x, int length) throws SQLException{}
		public void updateAsciiStream(String columnName, InputStream x, int length) throws SQLException{}
		public void updateBigDecimal(int columnIndex, BigDecimal x) throws SQLException{}
		public void updateBigDecimal(String columnName, BigDecimal x) throws SQLException{}
		public void updateBinaryStream(int columnIndex, InputStream x, int length) throws SQLException{}
		public void updateBinaryStream(String columnName, InputStream x, int length) throws SQLException{}
		public void updateBlob(int columnIndex, Blob x) throws SQLException{}
		public void updateBlob(String columnName, Blob x) throws SQLException{}
		public void updateBoolean(int columnIndex, boolean x) throws SQLException{}
		public void updateBoolean(String columnName, boolean x) throws SQLException{}
		public void updateByte(int columnIndex, byte x) throws SQLException{}
		public void updateByte(String columnName, byte x) throws SQLException{}
		public void updateBytes(int columnIndex, byte[] x) throws SQLException{}
		public void updateBytes(String columnName, byte[] x) throws SQLException{}
		public void updateCharacterStream(int columnIndex, Reader x, int length) throws SQLException{}
		public void updateCharacterStream(String columnName, Reader reader, int length) throws SQLException{}
		public void updateClob(int columnIndex, Clob x) throws SQLException{}
		public void updateClob(String columnName, Clob x) throws SQLException{}
		public void updateDate(int columnIndex, java.sql.Date x) throws SQLException{}
		public void updateDate(String columnName, java.sql.Date x) throws SQLException{}
		public void updateDouble(int columnIndex, double x) throws SQLException{}
		public void updateDouble(String columnName, double x) throws SQLException{}
		public void updateFloat(int columnIndex, float x) throws SQLException{}
		public void updateFloat(String columnName, float x) throws SQLException{}
		public void updateInt(int columnIndex, int x) throws SQLException{}
		public void updateInt(String columnName, int x) throws SQLException{}
		public void updateLong(int columnIndex, long x) throws SQLException{}
		public void updateLong(String columnName, long x) throws SQLException{}
		public void updateNull(int columnIndex) throws SQLException{}
		public void updateNull(String columnName) throws SQLException{}
		public void updateObject(int columnIndex, Object x, int scale) throws SQLException{}
		public void updateObject(int columnIndex, Object x) throws SQLException{}
		public void updateObject(String columnName, Object x, int scale) throws SQLException{}
		public void updateObject(String columnName, Object x) throws SQLException{}
		public void updateRef(int columnIndex, Ref x) throws SQLException {}
		public void updateRef(String columnName, Ref x) throws SQLException{}
		public void updateRow() throws SQLException{}
		public void updateShort(int columnIndex, short x) throws SQLException{}
		public void updateShort(String columnName, short x) throws SQLException{}
		public void updateString(int columnIndex, String x) throws SQLException{}
		public void updateString(String columnName, String x) throws SQLException{}
		public void updateTime(int columnIndex, Time x) throws SQLException{}
		public void updateTime(String columnName, Time x) throws SQLException{}
		public void updateTimestamp(int columnIndex, Timestamp x) throws SQLException{}
		public void updateTimestamp(String columnName, Timestamp x) throws SQLException {}
		public boolean wasNull() throws SQLException
		{
			// not implemented
			return false;
		}
		
	}
	
	private static class DummyMetaData implements ResultSetMetaData
	{

		private String[] columns;
		private String[] columnClasses;
		
		public DummyMetaData(String[] columns, String[] columnClasses) {
			this.columns = columns;
			this.columnClasses = columnClasses;
		}
		
		public int getColumnCount() throws SQLException
		{
			return columns.length;
		}

		public String getColumnName(int column) throws SQLException
		{
			return columns[column-1];
		}

		public String getCatalogName(int column) throws SQLException
		{
			// not implemented
			return null;
		}

		public String getColumnClassName(int column) throws SQLException
		{
			return columnClasses[column-1];
		}

		public int getColumnDisplaySize(int column) throws SQLException
		{
			return 0;
		}

		public String getColumnLabel(int column) throws SQLException
		{
			// not implemented
			return null;
		}

		public int getColumnType(int column) throws SQLException
		{
			// not implemented
			return 0;
		}

		public String getColumnTypeName(int column) throws SQLException
		{
			// not implemented
			return null;
		}

		public int getPrecision(int column) throws SQLException
		{
			// not implemented
			return 0;
		}

		public int getScale(int column) throws SQLException
		{
			// not implemented
			return 0;
		}

		public String getSchemaName(int column) throws SQLException
		{
			// not implemented
			return null;
		}

		public String getTableName(int column) throws SQLException
		{
			// not implemented
			return null;
		}

		public boolean isAutoIncrement(int column) throws SQLException
		{
			// not implemented
			return false;
		}

		public boolean isCaseSensitive(int column) throws SQLException
		{
			// not implemented
			return false;
		}

		public boolean isCurrency(int column) throws SQLException
		{
			// not implemented
			return false;
		}

		public boolean isDefinitelyWritable(int column) throws SQLException
		{
			// not implemented
			return false;
		}

		public int isNullable(int column) throws SQLException
		{
			// not implemented
			return 0;
		}

		public boolean isReadOnly(int column) throws SQLException
		{
			// not implemented
			return false;
		}

		public boolean isSearchable(int column) throws SQLException
		{
			// not implemented
			return false;
		}

		public boolean isSigned(int column) throws SQLException
		{
			// not implemented
			return false;
		}

		public boolean isWritable(int column) throws SQLException
		{
			// not implemented
			return false;
		}
		
	}
}
